<?php

namespace seraph_accel;

if( !defined( 'ABSPATH' ) )
	exit;

class Img
{
	static function GetInfoFromFile( $file, $ext = false )
	{
		$data = @file_get_contents( $file );
		return( $data !== false ? Img::GetInfoFromData( $data, $ext ) : null );
	}

	static function IsMimeRaster( $mime )
	{
		return( $mime != 'image/svg+xml' );
	}

	static function GetInfoFromData( $data, $ext = false )
	{
		$infoEx = $ext ? array() : null;

		if( gettype( $data ) == 'string' && @preg_match( '@<svg[>\\s]@i', $data ) && @preg_match( '@</svg>@i', $data ) )
		{
			$info = array( 'mime' => 'image/svg+xml', 'cx' => null, 'cy' => null );

			$doc = new \DOMDocument();
			if( !@$doc -> loadXML( $data, LIBXML_COMPACT | LIBXML_NOERROR | LIBXML_NONET | LIBXML_NOWARNING | LIBXML_PARSEHUGE ) )
				return( $info );

			$ndRoot = HtmlNd::FindByTag( $doc, 'svg' );
			if( !$ndRoot )
				return( $info );

			foreach( array( 'cx' => $ndRoot -> getAttribute( 'width' ), 'cy' => $ndRoot -> getAttribute( 'height' ) ) as $k => $v )
				$info[ $k ] = Gen::StrEndsWith( $v, 'px' ) ? intval( substr( $v, 0, -2 ) ) : null;

			return( $info );
		}

		if( !function_exists( 'getimagesizefromstring' ) )
			return( null );

		$info = @getimagesizefromstring( $data, $infoEx );
		if( !$info )
			return( null );

		$info = array( 'cx' => $info[ 0 ], 'cy' => $info[ 1 ], 'mime' => $info[ 'mime' ] );

		if( !$ext )
			return( $info );

		switch( $info[ 'mime' ] )
		{
		case 'image/gif':
			$info[ 'dpiX' ] = 96;
			$info[ 'dpiY' ] = 96;
			break;

		case 'image/png':

			$buf = array();

			$x = 0;
			$y = 0;
			$units = 0;

			for( $i = 0; $i < strlen( $data ); $i++ )
			{
				array_push( $buf, ord( $data[ $i ] ) );
				if( count( $buf ) > 13 )
					array_shift( $buf );
				if( count( $buf ) < 13 )
					continue;

				if( $buf[ 0 ] == ord( 'p' ) &&
					$buf[ 1 ] == ord( 'H' ) &&
					$buf[ 2 ] == ord( 'Y' ) &&
					$buf[ 3 ] == ord( 's' ) )
				{
					$x = ( $buf[ 4 ] << 24 ) + ( $buf[ 5 ] << 16 ) + ( $buf[ 6 ] << 8 ) + $buf[ 7 ];
					$y = ( $buf[ 8 ] << 24 ) + ( $buf[ 9 ] << 16 ) + ( $buf[ 10 ] << 8 ) + $buf[ 11 ];
					$units = $buf[ 12 ];
					break;
				}
			}

			switch( $units )
			{
			case 0:
				$info[ 'dpiX' ] = 96;
				$info[ 'dpiY' ] = intval( round( $info[ 'dpiX' ] * ( $x && $y ? ( $x / $y ) : 1 ) ) );
				break;

			case 1:
				$info[ 'dpiX' ] = intval( round( $x * 0.0254 ) );
				$info[ 'dpiY' ] = intval( round( $y * 0.0254 ) );
				break;
			}

			break;

		case 'image/tiff':
			break;

		case 'image/x-ms-bmp':
		case 'image/bmp':

			$data = substr( $data, 14 + 24, 8 );
			$info[ 'dpiX' ] = intval( @hexdec( @bin2hex( strrev( substr( $data, 0, 4 ) ) ) ) * 0.0254 );
			$info[ 'dpiY' ] = intval( @hexdec( @bin2hex( strrev( substr( $data, 4, 4 ) ) ) ) * 0.0254 );
			break;

		case 'image/jpeg':

			if( !(isset($infoEx[ 'APP0' ])?$infoEx[ 'APP0' ]:null) )
				$infoEx[ 'APP0' ] = substr( $data, 6, 20 - 6 );

			$data = @bin2hex( substr( $infoEx[ 'APP0' ], 8, 4 ) );

			$x = @hexdec( substr( $data, 0, 4 ) );
			$y = @hexdec( substr( $data, 4, 4 ) );
			$units = @hexdec( @bin2hex( substr( $infoEx[ 'APP0' ], 7, 1 ) ) );

			switch( $units )
			{
			case 0:
				$info[ 'dpiX' ] = 96;
				$info[ 'dpiY' ] = intval( round( $info[ 'dpiX' ] * $x / $y ) );
				break;

			case 1:
				$info[ 'dpiX' ] = $x;
				$info[ 'dpiY' ] = $y;
				break;

			case 2:
				$info[ 'dpiX' ] = intval( round( $x * 2.54 ) );
				$info[ 'dpiY' ] = intval( round( $y * 2.54 ) );
				break;
			}

			break;
		}

		return( $info );
	}

	static function GetData( $h, $mimeType, $quality = null, $file = null )
	{
		Img::GetDataEx( $res, $h, $mimeType, $quality, $file );
		return( $res );
	}

	static function GetDataEx( &$res, $h, $mimeType, $quality = null, $file = null )
	{
		if( !$h )
			return( Gen::E_INVALIDARG );

		$fn = null;

		$fileTmp = false;
		if( !$file )
		{
			$file = @tempnam( @sys_get_temp_dir(), '' );
			$fileTmp = true;
		}

		switch( $mimeType )
		{
		case 'image/gif':
			$fn = 'imagegif';
			$args = array( $h, $file );
			break;

		case 'image/jpeg':
			$fn = 'imagejpeg';
			$args = array( $h, $file, $quality === null ? -1 : $quality );
			break;

		case 'image/png':

			if( function_exists( 'imageistruecolor' ) && !@imageistruecolor( $h ) )
				@imagetruecolortopalette( $h, false, @imagecolorstotal( $h ) );

			$fn = 'imagepng';
			$args = array( $h, $file );
			break;

		case 'image/webp':
			if( !function_exists( 'imageistruecolor' ) )
				return( Gen::E_UNSUPPORTED );

			if( !@imageistruecolor( $h ) )
			{

				if( !@imagepalettetotruecolor( $h ) )
					return( Gen::E_FAIL );

			}

			$fn = 'imagewebp';
			$args = array( $h, $file, $quality === null ? -1 : $quality );
			break;

		case 'image/avif':
			if( !function_exists( 'imageistruecolor' ) )
				return( Gen::E_UNSUPPORTED );

			if( !@imageistruecolor( $h ) && !@imagepalettetotruecolor( $h ) )
				return( Gen::E_FAIL );

			$fn = 'imageavif';
			$args = array( $h, $file, $quality === null ? -1 : $quality );
			break;
		}

		if( !$fn )
		{
			if( $fileTmp )
				@unlink( $file );
			return( Gen::E_UNSUPPORTED );
		}

		if( !function_exists( $fn ) )
			return( Gen::E_UNSUPPORTED );

		$res = Gen::CallFunc( $fn, $args );
		if( !$fileTmp )
			return( $res ? Gen::S_OK : Gen::E_FAIL );

		$data = @file_get_contents( $file );
		@unlink( $file );

		if( !$res )
			return( Gen::E_FAIL );

		$res = $data;
		return( Gen::S_OK );
	}

	static function CreateFromData( $data )
	{
		Img::CreateFromDataEx( $res, $data );
		return( $res );
	}

	static function CreateFromDataEx( &$h, $data )
	{
		if( !$data )
			return( Gen::E_INVALIDARG );

		if( !function_exists( 'imagecreatefromstring' ) )
			return( Gen::E_UNSUPPORTED );

		$h = @imagecreatefromstring( $data );
		if( $h === false )
			return( Gen::E_FAIL );

		if( function_exists( 'imagealphablending' ) && function_exists( 'imagesavealpha' ) )
		{
			@imagealphablending( $h, false );
			@imagesavealpha( $h, true );
		}

		return( Gen::S_OK );
	}

	static function CreateCopyResample( $h, $sizeDst, $rcSrc = null, $rcDst = null, $bgClr = null )
	{
		if( !$sizeDst[ 'cx' ] || !$sizeDst[ 'cy' ] )
			return( null );

		$hNew = @imagecreatetruecolor( $sizeDst[ 'cx' ], $sizeDst[ 'cy' ] );
		if( $hNew === false )
			return( null );

		if( !$rcDst )
			$rcDst = array( 'x' => 0, 'y' => 0, 'cx' => $sizeDst[ 'cx' ], 'cy' => $sizeDst[ 'cy' ] );

		imagesavealpha( $hNew, true );
		imagefill( $hNew, 0, 0, imagecolorallocatealpha( $hNew, 0, 0, 0, 127 ) );

		if( $bgClr !== null )
			imagefilledrectangle( $hNew, 0, 0, $sizeDst[ 'cx' ] - 1, $sizeDst[ 'cy' ] - 1, imagecolorallocate( $hNew, ( $bgClr >> 16 ) & 0xFF, ( $bgClr >> 8 ) & 0xFF, $bgClr & 0xFF ) );

		if( function_exists( 'imageantialias' ) )
			imageantialias( $hNew, true );

		if( !@imagecopyresampled( $hNew, $h,
			$rcDst[ 'x' ], $rcDst[ 'y' ],
			$rcSrc[ 'x' ], $rcSrc[ 'y' ],
			$rcDst[ 'cx' ], $rcDst[ 'cy' ],
			$rcSrc[ 'cx' ], $rcSrc[ 'cy' ] ) )
		{
			imagedestroy( $hNew );
			return( null );
		}

		return( $hNew );
	}

	static function IsDataPngAnimated( $data )
	{

		$pos = strpos( $data, 'IDAT' );
		return( $pos !== false && strpos( substr( $data, 0, $pos ), 'acTL' ) !== false );
	}

	static function IsDataGifAnimated( $data )
	{

		for( $n = 0, $i = 0; $n < 2; $n++ )
		{
			if( !preg_match( '@\x00\x21\xF9\x04.{4}\x00(?:\x2C|\x21)@s', $data, $m, PREG_OFFSET_CAPTURE, $i ) )
				break;
			$i = $m[ 0 ][ 1 ] + strlen( $m[ 0 ][ 0 ] );
		}

		return( $n > 1 );
	}

	static function ConvertData( $data, $mimeType, $quality = null, $file = null )
	{
		Img::ConvertDataEx( $res, $data, $mimeType, $quality, $file );
		return( $res );
	}

	static function ConvertDataEx( &$res, $data, $mimeType, $quality = null, $file = null )
	{
		$type = substr( $mimeType, strlen( 'image/' ) );
		if( function_exists( 'image' . $type ) )
		{
			$hr = Img::CreateFromDataEx( $img, $data );
			if( Gen::HrFail( $hr ) )
				return( $hr );

			$hr = Img::GetDataEx( $res, $img, $mimeType, $quality, $file );
			if( $img )
				@imagedestroy( $img );
			return( $hr );
		}

		if( !class_exists( 'Imagick' ) )
			return( Gen::E_UNSUPPORTED );

		if( !$data )
			return( Gen::E_INVALIDARG );

		$img = new \Imagick();

		try
		{
			if( !$img -> readImageBlob( $data ) )
				return( Gen::E_FAIL );
		}
		catch( \Exception $e )
		{
			return( Gen::E_FAIL );
		}

		try
		{
			if( !$img -> setCompressionQuality( $quality === null ? 75 : $quality ) )
				return( Gen::E_FAIL );
		}
		catch( \Exception $e )
		{
			return( Gen::E_FAIL );
		}

		try
		{
			if( !$img -> setImageFormat( $type ) )
				return( Gen::E_UNSUPPORTED );
		}
		catch( \Exception $e )
		{
			return( Gen::E_UNSUPPORTED );
		}

		if( $file )
		{
			try
			{
				if( !$img -> writeImage( $file ) )
					return( Gen::E_FAIL );
				$res = true;
			}
			catch( \Exception $e )
			{
				return( Gen::E_FAIL );
			}
		}
		else
		{
			try
			{
				$res = $img -> getImageBlob();
			}
			catch( \Exception $e )
			{
				return( Gen::E_FAIL );
			}
		}

		return( Gen::S_OK );
	}
}

